Module: App::Command::StubbedRunInstanceMethod
- Defined in:
- lib/has_helpers/app/command.rb
Instance Method Summary collapse
-
#run ⇒ Object
Provides a default method which can serve as the super method for the masking method.
Instance Method Details
#run ⇒ Object
Provides a default method which can serve as the super method for the masking method.
64 65 66 |
# File 'lib/has_helpers/app/command.rb', line 64 def run(...) raise ::NotImplementedError end |