Module: HasHelpers::ActiveRecord::NestedResource::PrependedMethods

Defined in:
lib/has_helpers/active_record/nested_resource.rb

Instance Method Summary collapse

Instance Method Details

#nameObject



47
48
49
50
51
52
53
54
# File 'lib/has_helpers/active_record/nested_resource.rb', line 47

def name
  super
rescue
  # Rescuing is expensive, but there is no other accurate solution.
  # Maybe we should avoid falling back to this case by having each
  # nested resource define name, and allowing this case to fail.
  self.class.name.titleize
end