Exception: HasHelpers::Cards::Importer::AuditError

Inherits:
StandardError
  • Object
show all
Defined in:
app/importers/has_helpers/cards/importer.rb

Overview

Raised when auditing detects a breaking drift between DSL (code) and DB. We use this error to stop the import and force devs to write a data migration.