Class: HasHelpers::Resource::TagPresenter

Inherits:
BasePresenter show all
Defined in:
app/presenters/has_helpers/resource/tag_presenter.rb

Instance Attribute Summary collapse

Attributes inherited from BasePresenter

#errors

Method Summary

Methods inherited from BasePresenter

#==, #error_on, #initialize, wrap, wrap_with_options

Methods included from Attributes

included

Constructor Details

This class inherits a constructor from HasHelpers::BasePresenter

Dynamic Method Handling

This class handles dynamic methods through the method_missing method in the class HasHelpers::BasePresenter

Instance Attribute Details

#templateObject

use I can think of right now is using a related_entity template (value must be set to an instance of Resource::InfoPresenter)



12
13
14
# File 'app/presenters/has_helpers/resource/tag_presenter.rb', line 12

def template
  @template || detect_template_from_value
end

#valueObject



16
17
18
# File 'app/presenters/has_helpers/resource/tag_presenter.rb', line 16

def value
  wrapped_object
end