Class: HasHelpers::FixedExpirationTimeCacheSessionStore
- Inherits:
-
ActionDispatch::Session::CacheStore
- Object
- ActionDispatch::Session::CacheStore
- HasHelpers::FixedExpirationTimeCacheSessionStore
- Defined in:
- lib/has_helpers/fixed_expiration_time_cache_session_store.rb
Overview
ActionDispatch::Session::CacheStore custom store where the expired date no longer changes after creation
Instance Method Summary collapse
Instance Method Details
#write_session(env, sid, session_data, options) ⇒ Object
6 7 8 9 10 11 12 13 |
# File 'lib/has_helpers/fixed_expiration_time_cache_session_store.rb', line 6 def write_session(env, sid, session_data, ) # Only write the session to the store if it's a new session unless session_already_exists?(sid) super(env, sid, session_data, ) end sid end |