Class: HasHelpers::Resource::TagPresenter
- Inherits:
-
BasePresenter
- Object
- BasePresenter
- HasHelpers::Resource::TagPresenter
- Defined in:
- app/presenters/has_helpers/resource/tag_presenter.rb
Instance Attribute Summary collapse
-
#template ⇒ Object
use I can think of right now is using a related_entity template (value must be set to an instance of Resource::InfoPresenter).
- #value ⇒ Object
Attributes inherited from BasePresenter
Method Summary
Methods inherited from BasePresenter
#==, #error_on, #initialize, wrap, wrap_with_options
Methods included from Attributes
Constructor Details
This class inherits a constructor from HasHelpers::BasePresenter
Dynamic Method Handling
This class handles dynamic methods through the method_missing method in the class HasHelpers::BasePresenter
Instance Attribute Details
#template ⇒ Object
use I can think of right now is using a related_entity template (value must be set to an instance of Resource::InfoPresenter)
12 13 14 |
# File 'app/presenters/has_helpers/resource/tag_presenter.rb', line 12 def template @template || detect_template_from_value end |
#value ⇒ Object
16 17 18 |
# File 'app/presenters/has_helpers/resource/tag_presenter.rb', line 16 def value wrapped_object end |